@@ -569,18 +569,66 @@ const char *entangle_camera_get_port(EntangleCamera *cam)
     return priv->port;
 }
 
+struct EntangleCameraProgressData {
+    EntangleCamera *cam;
+    enum {
+        ENTANGLE_CAMERA_PROGRESS_START,
+        ENTANGLE_CAMERA_PROGRESS_UPDATE,
+        ENTANGLE_CAMERA_PROGRESS_STOP
+    } op;
+    float value;
+    char *msg;
+};
+
+static gboolean entangle_camera_progress_idle(gpointer opaque)
+{
+    struct EntangleCameraProgressData *data = opaque;
+
+    if (data->cam->priv->progress) {
+        switch (data->op) {
+        case ENTANGLE_CAMERA_PROGRESS_START:
+            entangle_progress_start(data->cam->priv->progress,
+                                    data->value,
+                                    data->msg);
+            break;
+
+        case ENTANGLE_CAMERA_PROGRESS_UPDATE:
+            entangle_progress_update(data->cam->priv->progress,
+                                     data->value);
+            break;
+
+        case ENTANGLE_CAMERA_PROGRESS_STOP:
+            entangle_progress_stop(data->cam->priv->progress);
+            break;
+
+        default:
+            break;
+        }
+    }
+
+    if (data->op == ENTANGLE_CAMERA_PROGRESS_START)
+        g_free(data->msg);
+    g_object_unref(data->cam);
+    g_free(data);
+
+    return FALSE;
+}
 
 #ifdef HAVE_GPHOTO25
 static unsigned int do_entangle_camera_progress_start(GPContext *ctx G_GNUC_UNUSED,
                                                       float target,
                                                       const char *msg,
-                                                      void *data)
+                                                      void *opaque)
 {
-    EntangleCamera *cam = data;
-    EntangleCameraPrivate *priv = cam->priv;
+    EntangleCamera *cam = opaque;
+    struct EntangleCameraProgressData *data = g_new0(struct EntangleCameraProgressData, 1);
 
-    if (priv->progress)
-        entangle_progress_start(priv->progress, target, msg);
+    data->cam = g_object_ref(cam);
+    data->op = ENTANGLE_CAMERA_PROGRESS_START;
+    data->value = target;
+    data->msg = g_strdup(msg);
+
+    g_idle_add(entangle_camera_progress_idle, data);
 
     return 0; /* XXX what is this actually useful for ? */
 }
@@ -589,16 +637,17 @@ static unsigned int do_entangle_camera_progress_start(GPContext *ctx G_GNUC_UNUS
                                                       float target,
                                                       const char *format,
                                                       va_list args,
-                                                      void *data)
+                                                      void *opaque)
 {
-    EntangleCamera *cam = data;
-    EntangleCameraPrivate *priv = cam->priv;
+    EntangleCamera *cam = opaque;
+    struct EntangleCameraProgressData *data = g_new0(struct EntangleCameraProgressData, 1);
 
-    if (priv->progress) {
-        char *str = g_strdup_vprintf(format, args);
-        entangle_progress_start(priv->progress, target, str);
-        g_free(str);
-    }
+    data->cam = g_object_ref(cam);
+    data->op = ENTANGLE_CAMERA_PROGRESS_START;
+    data->value = target;
+    data->msg = g_strdup_vprintf(format, args);
+
+    g_idle_add(entangle_camera_progress_idle, data);
 
     return 0; /* XXX what is this actually useful for ? */
 }
@@ -607,24 +656,29 @@ static unsigned int do_entangle_camera_progress_start(GPContext *ctx G_GNUC_UNUS
 static void do_entangle_camera_progress_update(GPContext *ctx G_GNUC_UNUSED,
                                                unsigned int id G_GNUC_UNUSED,
                                                float current,
-                                               void *data)
+                                               void *opaque)
 {
-    EntangleCamera *cam = data;
-    EntangleCameraPrivate *priv = cam->priv;
+    EntangleCamera *cam = opaque;
+    struct EntangleCameraProgressData *data = g_new0(struct EntangleCameraProgressData, 1);
 
-    if (priv->progress)
-        entangle_progress_update(priv->progress, current);
+    data->cam = g_object_ref(cam);
+    data->op = ENTANGLE_CAMERA_PROGRESS_UPDATE;
+    data->value = current;
+
+    g_idle_add(entangle_camera_progress_idle, data);
 }
 
 static void do_entangle_camera_progress_stop(GPContext *ctx G_GNUC_UNUSED,
                                              unsigned int id G_GNUC_UNUSED,
-                                             void *data)
+                                             void *opaque)
 {
-    EntangleCamera *cam = data;
-    EntangleCameraPrivate *priv = cam->priv;
+    EntangleCamera *cam = opaque;
+    struct EntangleCameraProgressData *data = g_new0(struct EntangleCameraProgressData, 1);
 
-    if (priv->progress)
-        entangle_progress_stop(priv->progress);
+    data->cam = g_object_ref(cam);
+    data->op = ENTANGLE_CAMERA_PROGRESS_STOP;
+
+    g_idle_add(entangle_camera_progress_idle, data);
 }
 
 static void entangle_camera_reset_last_error(EntangleCamera *cam)
@@ -869,7 +923,7 @@ gboolean entangle_camera_disconnect(EntangleCamera *cam,
     gp_camera_unref(priv->cam);
     priv->cam = NULL;
     priv->hasViewfinder = FALSE;
-    
+
     priv->hasCapture = priv->hasPreview = priv->hasSettings = FALSE;
 
     ret = TRUE;
@@ -1780,6 +1834,12 @@ static gboolean entangle_str_equal_null(gchar *a, gchar *b)
 }
 
 
+/*
+ * XXX this method causes signals to be emitted from controls
+ * in non-main threads, if triggered via an _async() method.
+ * Investigate if we can fix this to always run in main thread
+ * to simplify thread safety for GTK frontend
+ */
 static gboolean do_load_controls(EntangleCamera *cam,
                                  const char *path,
                                  CameraWidget *widget,
@@ -2277,7 +2337,7 @@ gboolean entangle_camera_set_viewfinder(EntangleCamera *cam,
                     gp_port_result_as_string(err), err);
         goto cleanup;
     }
-    
+
     if ((err = gp_camera_set_config(priv->cam,
                                     priv->widgets,
                                     priv->ctx)) != GP_OK) {
@@ -2415,7 +2475,7 @@ gboolean entangle_camera_autofocus(EntangleCamera *cam,
                     gp_port_result_as_string(err), err);
         goto cleanup;
     }
-    
+
     if ((err = gp_camera_set_config(priv->cam,
                                     priv->widgets,
                                     priv->ctx)) != GP_OK) {
@@ -2541,7 +2601,7 @@ gboolean entangle_camera_manualfocus(EntangleCamera *cam,
                     gp_port_result_as_string(err), err);
         goto cleanup;
     }
-    
+
     if ((err = gp_camera_set_config(priv->cam,
                                     priv->widgets,
                                     priv->ctx)) != GP_OK) {
@@ -2747,16 +2807,14 @@ static GMount *entangle_device_manager_find_mount(EntangleCamera *cam,
  * CPU burn loop in a bg thread from DBus messages. So we
  * have made this static for now
  */
-static GStaticMutex monitorLock = G_STATIC_MUTEX_INIT;
-static GVolumeMonitor *monitor;
 
 gboolean entangle_camera_is_mounted(EntangleCamera *cam)
 {
     g_return_val_if_fail(ENTANGLE_IS_CAMERA(cam), FALSE);
+    static GVolumeMonitor *monitor;
 
     GMount *mount;
     gboolean ret;
-    g_static_mutex_lock(&monitorLock);
     if (!monitor)
         monitor = g_volume_monitor_get();
     mount = entangle_device_manager_find_mount(cam, monitor);
@@ -2767,7 +2825,6 @@ gboolean entangle_camera_is_mounted(EntangleCamera *cam)
     } else {
         ret = FALSE;
     }
-    g_static_mutex_unlock(&monitorLock);
 
     return ret;
 }
