@@ -26,7 +26,8 @@
 #include "entangle-pixbuf.h"
 
 /* Performs rotation of the thunbnail based on embedded metadata */
-GdkPixbuf *entangle_pixbuf_auto_rotate(GdkPixbuf *src)
+GdkPixbuf *entangle_pixbuf_auto_rotate(GdkPixbuf *src,
+                                       GExiv2Metadata *metadata)
 {
     GdkPixbuf *dest = gdk_pixbuf_apply_embedded_orientation(src);
     GdkPixbuf *temp;
@@ -34,19 +35,25 @@ GdkPixbuf *entangle_pixbuf_auto_rotate(GdkPixbuf *src)
     ENTANGLE_DEBUG("Auto-rotate %p %p\n", src, dest);
 
     if (dest == src) {
-        const char *orientationstr;
         int transform = 0;
-        orientationstr = gdk_pixbuf_get_option(src, "tEXt::Entangle::Orientation");
 
-        /* If not option, then try the gobject data slot */
-        if (!orientationstr)
-            orientationstr = g_object_get_data(G_OBJECT(src),
-                                               "tEXt::Entangle::Orientation");
+        g_object_unref(dest);
 
-        if (orientationstr)
-            transform = (int)g_ascii_strtoll(orientationstr, NULL, 10);
+        if (metadata) {
+            transform = gexiv2_metadata_get_orientation(metadata);
+        } else {
+            const char *orientationstr = gdk_pixbuf_get_option(src, "tEXt::Entangle::Orientation");
 
-        ENTANGLE_DEBUG("Auto-rotate %s\n", orientationstr);
+            /* If not option, then try the gobject data slot */
+            if (!orientationstr)
+                orientationstr = g_object_get_data(G_OBJECT(src),
+                                                   "tEXt::Entangle::Orientation");
+
+            if (orientationstr)
+                transform = (int)g_ascii_strtoll(orientationstr, NULL, 10);
+
+            ENTANGLE_DEBUG("Auto-rotate %s\n", orientationstr);
+        }
 
         /* Apply the actual transforms, which involve rotations and flips.
            The meaning of orientation values 1-8 and the required transforms
@@ -90,8 +97,6 @@ GdkPixbuf *entangle_pixbuf_auto_rotate(GdkPixbuf *src)
 
     }
 
-    g_object_unref(src);
-
     return dest;
 }
 
@@ -102,19 +107,23 @@ static gboolean entangle_pixbuf_is_raw(EntangleImage *image)
         ".crw", ".erf", ".mrw", ".raw", ".rw2", ".raf", NULL
     };
     const char **tmp;
-    const char *filename = entangle_image_get_filename(image);
+    char *filename = g_utf8_strdown(entangle_image_get_filename(image), -1);
+    gboolean ret = TRUE;
 
     tmp = extlist;
     while (*tmp) {
         const char *ext = *tmp;
 
         if (g_str_has_suffix(filename, ext))
-            return TRUE;
+            goto cleanup;
 
         tmp++;
     }
 
-    return FALSE;
+    ret = FALSE;
+ cleanup:
+    g_free(filename);
+    return ret;
 }
 
 
@@ -137,6 +146,16 @@ static GdkPixbuf *entangle_pixbuf_open_image_master_raw(EntangleImage *image)
         goto cleanup;
     }
 
+    /*
+      Use default camera parameters to display image
+      Real work needs to done by a RAW developer anyway
+    */
+    raw->params.use_camera_wb = 1;
+    raw->params.use_auto_wb = 1;
+    raw->params.use_camera_matrix = 1;
+    raw->params.user_qual = 0;
+    raw->params.fbdd_noiserd = 1;
+
     ENTANGLE_DEBUG("Open raw %s", entangle_image_get_filename(image));
     if ((ret = libraw_open_file(raw, entangle_image_get_filename(image))) != 0) {
         ENTANGLE_DEBUG("Failed to open raw file: %s",
@@ -179,7 +198,9 @@ static GdkPixbuf *entangle_pixbuf_open_image_master_raw(EntangleImage *image)
 }
 
 
-static GdkPixbuf *entangle_pixbuf_open_image_master_gdk(EntangleImage *image)
+static GdkPixbuf *entangle_pixbuf_open_image_master_gdk(EntangleImage *image,
+                                                        GExiv2Metadata *metadata,
+                                                        gboolean applyOrientation)
 {
     GdkPixbuf *master;
     GdkPixbuf *result;
@@ -190,19 +211,32 @@ static GdkPixbuf *entangle_pixbuf_open_image_master_gdk(EntangleImage *image)
     if (!master)
         return NULL;
 
-    result = gdk_pixbuf_apply_embedded_orientation(master);
+    if (applyOrientation) {
+        result = entangle_pixbuf_auto_rotate(master, metadata);
+        g_object_unref(master);
+    } else {
+        GExiv2Orientation orient = gexiv2_metadata_get_orientation(metadata);
+        /* gdk_pixbuf_save doesn't update internal options and there
+           is no set_option method, so abuse gobject data slots :-( */
+        g_object_set_data_full(G_OBJECT(master),
+                               "tEXt::Entangle::Orientation",
+                               g_strdup_printf("%d", orient),
+                               g_free);
+        result = master;
+    }
 
-    g_object_unref(master);
     return result;
 }
 
 
-static GdkPixbuf *entangle_pixbuf_open_image_master(EntangleImage *image)
+static GdkPixbuf *entangle_pixbuf_open_image_master(EntangleImage *image,
+                                                    GExiv2Metadata *metadata,
+                                                    gboolean applyOrientation)
 {
     if (entangle_pixbuf_is_raw(image))
         return entangle_pixbuf_open_image_master_raw(image);
     else
-        return entangle_pixbuf_open_image_master_gdk(image);
+        return entangle_pixbuf_open_image_master_gdk(image, metadata, applyOrientation);
 }
 
 
@@ -256,10 +290,86 @@ entangle_pixbuf_get_closest_preview(GExiv2PreviewProperties **proplist,
 }
 
 
+static GdkPixbuf *entangle_pixbuf_open_image_preview_raw(EntangleImage *image,
+                                                         GExiv2Metadata *metadata,
+                                                         gboolean applyOrientation)
+{
+    GdkPixbuf *result = NULL;
+    GdkPixbufLoader *loader = gdk_pixbuf_loader_new();
+    libraw_data_t *raw = libraw_init(0);
+    libraw_processed_image_t *img = NULL;
+    int ret;
+
+    if (!raw) {
+        ENTANGLE_DEBUG("Failed to initialize libraw");
+        goto cleanup;
+    }
+
+    ENTANGLE_DEBUG("Open preview raw %s", entangle_image_get_filename(image));
+    if ((ret = libraw_open_file(raw, entangle_image_get_filename(image))) != 0) {
+        ENTANGLE_DEBUG("Failed to open preview raw file: %s",
+                       libraw_strerror(ret));
+        goto cleanup;
+    }
+
+    ENTANGLE_DEBUG("Unpack preview raw %s", entangle_image_get_filename(image));
+    if ((ret = libraw_unpack_thumb(raw)) != 0) {
+        ENTANGLE_DEBUG("Failed to unpack preview raw file: %s",
+                       libraw_strerror(ret));
+        goto cleanup;
+    }
+
+    ENTANGLE_DEBUG("Adjust preview raw %s", entangle_image_get_filename(image));
+    if ((ret = libraw_adjust_sizes_info_only(raw)) != 0) {
+        ENTANGLE_DEBUG("Failed to adjust preview raw file: %s",
+                       libraw_strerror(ret));
+        goto cleanup;
+    }
+
+    ENTANGLE_DEBUG("Make preview mem %s", entangle_image_get_filename(image));
+    if ((img = libraw_dcraw_make_mem_thumb(raw, &ret)) == NULL) {
+        ENTANGLE_DEBUG("Failed to extract preview raw file: %s",
+                       libraw_strerror(ret));
+        goto cleanup;
+    }
+
+    gdk_pixbuf_loader_write(loader, img->data, img->data_size, NULL);
+    result = gdk_pixbuf_loader_get_pixbuf(loader);
+
+    if (result) {
+        g_object_ref(result);
+        if (applyOrientation) {
+            GdkPixbuf *tmp = entangle_pixbuf_auto_rotate(result, metadata);
+            g_object_unref(result);
+            result = tmp;
+        } else {
+            GExiv2Orientation orient = gexiv2_metadata_get_orientation(metadata);
+            /* gdk_pixbuf_save doesn't update internal options and there
+               is no set_option method, so abuse gobject data slots :-( */
+            g_object_set_data_full(G_OBJECT(result),
+                                   "tEXt::Entangle::Orientation",
+                                   g_strdup_printf("%d", orient),
+                                   g_free);
+        }
+    }
+
+
+ cleanup:
+    if (img)
+        libraw_dcraw_clear_mem(img);
+
+    libraw_close(raw);
+
+    gdk_pixbuf_loader_close(loader, NULL);
+
+    return result;
+}
+
+
 static GdkPixbuf *entangle_pixbuf_open_image_preview_exiv(EntangleImage *image,
-                                                          guint minSize)
+                                                          guint minSize,
+                                                          GExiv2Metadata *metadata)
 {
-    GExiv2Metadata *metadata = gexiv2_metadata_new();
     GExiv2PreviewImage *preview = NULL;
     GExiv2PreviewProperties **props;
     GExiv2PreviewProperties *best;
@@ -271,11 +381,6 @@ static GdkPixbuf *entangle_pixbuf_open_image_preview_exiv(EntangleImage *image,
 
     ENTANGLE_DEBUG("Opening preview %s", entangle_image_get_filename(image));
 
-    if (!gexiv2_metadata_open_path(metadata, entangle_image_get_filename(image), NULL)) {
-        ENTANGLE_DEBUG("Unable to load exiv2 data");
-        goto cleanup;
-    }
-
     props = gexiv2_metadata_get_preview_properties(metadata);
 
     if (minSize)
@@ -315,59 +420,84 @@ static GdkPixbuf *entangle_pixbuf_open_image_preview_exiv(EntangleImage *image,
                            g_strdup_printf("%d", orient),
                            g_free);
 
-    result = entangle_pixbuf_auto_rotate(master);
-
  cleanup:
     if (loader)
         g_object_unref(loader);
     if (preview)
         g_object_unref(preview);
-    if (metadata)
-        g_object_unref(metadata);
     return result;
 }
 
 
-static GdkPixbuf *entangle_pixbuf_open_image_preview(EntangleImage *image)
+static GdkPixbuf *entangle_pixbuf_open_image_preview(EntangleImage *image,
+                                                     GExiv2Metadata *metadata,
+                                                     gboolean applyOrientation)
 {
+    GdkPixbuf *result = NULL;
     if (entangle_pixbuf_is_raw(image)) {
-        GdkPixbuf *result = entangle_pixbuf_open_image_preview_exiv(image, 256);
+        result = entangle_pixbuf_open_image_preview_raw(image, metadata, applyOrientation);
+        if (!result && metadata)
+            result = entangle_pixbuf_open_image_preview_exiv(image, 256, metadata);
         if (!result)
             result = entangle_pixbuf_open_image_master_raw(image);
-        return result;
     } else {
-        return entangle_pixbuf_open_image_master_gdk(image);
+        result = entangle_pixbuf_open_image_master_gdk(image, metadata, applyOrientation);
     }
+    return result;
 }
 
 
-static GdkPixbuf *entangle_pixbuf_open_image_thumbnail(EntangleImage *image)
+static GdkPixbuf *entangle_pixbuf_open_image_thumbnail(EntangleImage *image,
+                                                       GExiv2Metadata *metadata,
+                                                       gboolean applyOrientation)
 {
-    GdkPixbuf *result = entangle_pixbuf_open_image_preview_exiv(image, 128);
+    GdkPixbuf *result = NULL;
+    if (entangle_pixbuf_is_raw(image))
+        result = entangle_pixbuf_open_image_preview_raw(image, metadata, applyOrientation);
+    if (!result && metadata)
+        result = entangle_pixbuf_open_image_preview_exiv(image, 128, metadata);
     if (!result)
-        result = entangle_pixbuf_open_image_master(image);
+        result = entangle_pixbuf_open_image_master(image, metadata, applyOrientation);
     return result;
 }
 
 
 GdkPixbuf *entangle_pixbuf_open_image(EntangleImage *image,
-                                      EntanglePixbufImageSlot slot)
+                                      EntanglePixbufImageSlot slot,
+                                      gboolean applyOrientation,
+                                      GExiv2Metadata **metadata)
 {
     ENTANGLE_DEBUG("Open image %s %d", entangle_image_get_filename(image), slot);
+    GExiv2Metadata *themetadata = gexiv2_metadata_new();
+    GdkPixbuf *ret = NULL;
+
+    if (!gexiv2_metadata_open_path(themetadata, entangle_image_get_filename(image), NULL)) {
+        g_object_unref(themetadata);
+        themetadata = NULL;
+    }
+
     switch (slot) {
     case ENTANGLE_PIXBUF_IMAGE_SLOT_MASTER:
-        return entangle_pixbuf_open_image_master(image);
+        ret = entangle_pixbuf_open_image_master(image, themetadata, applyOrientation);
+        break;
 
     case ENTANGLE_PIXBUF_IMAGE_SLOT_PREVIEW:
-        return entangle_pixbuf_open_image_preview(image);
+        ret = entangle_pixbuf_open_image_preview(image, themetadata, applyOrientation);
+        break;
 
     case ENTANGLE_PIXBUF_IMAGE_SLOT_THUMBNAIL:
-        return entangle_pixbuf_open_image_thumbnail(image);
+        ret = entangle_pixbuf_open_image_thumbnail(image, themetadata, applyOrientation);
+        break;
 
     default:
         g_warn_if_reached();
-        return NULL;
+        break;
     }
+    if (metadata)
+        *metadata = themetadata;
+    else
+        g_object_unref(themetadata);
+    return ret;
 }
 
 /*
