@@ -92,22 +92,28 @@ static void do_update_control_finish(GObject *src,
 }
 
 
-static void do_refresh_control_entry(GObject *object,
-                                     GParamSpec *pspec G_GNUC_UNUSED,
-                                     gpointer data)
+static gboolean do_refresh_control_entry_idle(gpointer data)
 {
     GtkWidget *widget = GTK_WIDGET(data);
+    GObject *control = g_object_get_data(G_OBJECT(widget), "control");
     gchar *text;
-    gdk_threads_enter();
-    g_object_get(object, "value", &text, NULL);
+
+    g_object_get(control, "value", &text, NULL);
     if (GTK_IS_LABEL(widget))
         gtk_label_set_text(GTK_LABEL(widget), text);
     else
         gtk_entry_set_text(GTK_ENTRY(widget), text);
     g_free(text);
-    gdk_threads_leave();
+
+    return FALSE;
 }
 
+static void do_refresh_control_entry(GObject *object G_GNUC_UNUSED,
+                                     GParamSpec *pspec G_GNUC_UNUSED,
+                                     gpointer data)
+{
+    g_idle_add(do_refresh_control_entry_idle, data);
+}
 
 static void do_update_control_entry(GtkWidget *widget,
                                     GdkEventFocus *ev G_GNUC_UNUSED,
@@ -123,9 +129,7 @@ static void do_update_control_entry(GtkWidget *widget,
     text = gtk_entry_get_text(GTK_ENTRY(widget));
 
     ENTANGLE_DEBUG("entry [%s]", text);
-    gdk_threads_leave();
     g_object_set(control, "value", text, NULL);
-    gdk_threads_enter();
 
     entangle_camera_save_controls_async(priv->camera,
                                         NULL,
@@ -134,15 +138,13 @@ static void do_update_control_entry(GtkWidget *widget,
 }
 
 
-static void do_refresh_control_range(GObject *object,
-                                     GParamSpec *pspec G_GNUC_UNUSED,
-                                     gpointer data)
+static gboolean do_refresh_control_range_idle(gpointer data)
 {
     GtkWidget *widget = GTK_WIDGET(data);
+    GObject *control = g_object_get_data(G_OBJECT(widget), "control");
     gfloat val;
 
-    gdk_threads_enter();
-    g_object_get(object, "value", &val, NULL);
+    g_object_get(control, "value", &val, NULL);
     if (GTK_IS_LABEL(widget)) {
         gchar *text = g_strdup_printf("%0.02f", val);
         gtk_label_set_text(GTK_LABEL(widget), text);
@@ -150,7 +152,16 @@ static void do_refresh_control_range(GObject *object,
     } else {
         gtk_range_set_value(GTK_RANGE(widget), val);
     }
-    gdk_threads_leave();
+
+    return FALSE;
+}
+
+
+static void do_refresh_control_range(GObject *object G_GNUC_UNUSED,
+                                     GParamSpec *pspec G_GNUC_UNUSED,
+                                     gpointer data)
+{
+    g_idle_add(do_refresh_control_range_idle, data);
 }
 
 
@@ -166,9 +177,7 @@ static void do_update_control_range(GtkRange *widget G_GNUC_UNUSED,
     EntangleControlPanelPrivate *priv = panel->priv;
 
     ENTANGLE_DEBUG("range [%lf]", value);
-    gdk_threads_leave();
     g_object_set(control, "value", (float)value, NULL);
-    gdk_threads_enter();
 
     entangle_camera_save_controls_async(priv->camera,
                                         NULL,
@@ -177,28 +186,35 @@ static void do_update_control_range(GtkRange *widget G_GNUC_UNUSED,
 }
 
 
-static void do_refresh_control_combo(GObject *object,
-                                     GParamSpec *pspec G_GNUC_UNUSED,
-                                     gpointer data)
+static gboolean do_refresh_control_combo_idle(gpointer data)
 {
     GtkWidget *widget = GTK_WIDGET(data);
+    GObject *control = g_object_get_data(G_OBJECT(widget), "control");
     gchar *text;
 
-    gdk_threads_enter();
-    g_object_get(object, "value", &text, NULL);
+    g_object_get(control, "value", &text, NULL);
 
     if (GTK_IS_LABEL(widget)) {
         gtk_label_set_text(GTK_LABEL(widget), text);
     } else {
         int active = 0;
-        for (int n = 0 ; n < entangle_control_choice_entry_count(ENTANGLE_CONTROL_CHOICE(object)) ; n++) {
-            if (g_strcmp0(text, entangle_control_choice_entry_get(ENTANGLE_CONTROL_CHOICE(object), n)) == 0)
+        for (int n = 0 ; n < entangle_control_choice_entry_count(ENTANGLE_CONTROL_CHOICE(control)) ; n++) {
+            if (g_strcmp0(text, entangle_control_choice_entry_get(ENTANGLE_CONTROL_CHOICE(control), n)) == 0)
                 active = n;
         }
         gtk_combo_box_set_active(GTK_COMBO_BOX(widget), active);
     }
     g_free(text);
-    gdk_threads_leave();
+
+    return FALSE;
+}
+
+
+static void do_refresh_control_combo(GObject *object G_GNUC_UNUSED,
+                                     GParamSpec *pspec G_GNUC_UNUSED,
+                                     gpointer data)
+{
+    g_idle_add(do_refresh_control_combo_idle, data);
 }
 
 
@@ -218,9 +234,7 @@ static void do_update_control_combo(GtkComboBox *widget,
         gtk_tree_model_get(model, &iter, 0, &text, -1);
 
     ENTANGLE_DEBUG("combo [%s]", text);
-    gdk_threads_leave();
     g_object_set(control, "value", text, NULL);
-    gdk_threads_enter();
 
     g_free(text);
 
@@ -231,21 +245,28 @@ static void do_update_control_combo(GtkComboBox *widget,
 }
 
 
-static void do_refresh_control_toggle(GObject *object,
-                                      GParamSpec *pspec G_GNUC_UNUSED,
-                                      gpointer data)
+static gboolean do_refresh_control_toggle_idle(gpointer data)
 {
     GtkWidget *widget = GTK_WIDGET(data);
+    GObject *control = g_object_get_data(G_OBJECT(widget), "control");
     gboolean state;
 
-    gdk_threads_enter();
-    g_object_get(object, "value", &state, NULL);
+    g_object_get(control, "value", &state, NULL);
     if (GTK_IS_LABEL(widget))
         gtk_label_set_text(GTK_LABEL(widget), state ? _("On") : _("Off"));
     else
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
                                      state);
-    gdk_threads_leave();
+
+    return FALSE;
+}
+
+
+static void do_refresh_control_toggle(GObject *object G_GNUC_UNUSED,
+                                      GParamSpec *pspec G_GNUC_UNUSED,
+                                      gpointer data)
+{
+    g_idle_add(do_refresh_control_toggle_idle, data);
 }
 
 
@@ -261,9 +282,7 @@ static void do_update_control_toggle(GtkToggleButton *widget,
 
     active = gtk_toggle_button_get_active(widget);
     ENTANGLE_DEBUG("toggle [%d]", active);
-    gdk_threads_leave();
     g_object_set(control, "value", active, NULL);
-    gdk_threads_enter();
 
     entangle_camera_save_controls_async(priv->camera,
                                         NULL,
@@ -271,6 +290,27 @@ static void do_update_control_toggle(GtkToggleButton *widget,
                                         panel);
 }
 
+static gboolean do_update_control_readonly_idle(gpointer data)
+{
+    GtkWidget *widget = GTK_WIDGET(data);
+    GObject *control = g_object_get_data(G_OBJECT(widget), "control");
+    gboolean state;
+
+    g_object_get(control, "readonly", &state, NULL);
+    gtk_widget_set_sensitive(widget, !state);
+
+    return FALSE;
+}
+
+
+static void do_update_control_readonly(GObject *object G_GNUC_UNUSED,
+                                       GParamSpec *pspec G_GNUC_UNUSED,
+                                       gpointer data)
+{
+    g_idle_add(do_update_control_readonly_idle, data);
+}
+
+
 static void do_setup_control_group(EntangleControlPanel *panel,
                                    GtkVBox *box,
                                    EntangleControlGroup *grp)
@@ -306,6 +346,8 @@ static void do_setup_control_group(EntangleControlPanel *panel,
             if (entangle_control_get_readonly(control))
                 gtk_widget_set_sensitive(value, FALSE);
             gtk_container_add(GTK_CONTAINER(subbox), value);
+            g_signal_connect(control, "notify::readonly",
+                             G_CALLBACK(do_update_control_readonly), value);
         } else if (ENTANGLE_IS_CONTROL_CHOICE(control)) {
             GtkCellRenderer *cell;
             GtkListStore *store;
@@ -363,6 +405,8 @@ static void do_setup_control_group(EntangleControlPanel *panel,
                              G_CALLBACK(do_update_control_combo), panel);
             g_signal_connect(control, "notify::value",
                              G_CALLBACK(do_refresh_control_combo), value);
+            g_signal_connect(control, "notify::readonly",
+                             G_CALLBACK(do_update_control_readonly), value);
             gtk_container_add(GTK_CONTAINER(subbox), value);
         } else if (ENTANGLE_IS_CONTROL_DATE(control)) {
             GtkWidget *label;
@@ -412,6 +456,8 @@ static void do_setup_control_group(EntangleControlPanel *panel,
                              G_CALLBACK(do_update_control_range), panel);
             g_signal_connect(control, "notify::value",
                              G_CALLBACK(do_refresh_control_range), value);
+            g_signal_connect(control, "notify::readonly",
+                             G_CALLBACK(do_update_control_readonly), value);
             gtk_container_add(GTK_CONTAINER(subbox), value);
         } else if (ENTANGLE_IS_CONTROL_TEXT(control)) {
             GtkWidget *label;
@@ -434,6 +480,8 @@ static void do_setup_control_group(EntangleControlPanel *panel,
                              G_CALLBACK(do_update_control_entry), panel);
             g_signal_connect(control, "notify::value",
                              G_CALLBACK(do_refresh_control_entry), value);
+            g_signal_connect(control, "notify::readonly",
+                             G_CALLBACK(do_update_control_readonly), value);
             gtk_container_add(GTK_CONTAINER(subbox), value);
         } else if (ENTANGLE_IS_CONTROL_TOGGLE(control)) {
             GtkWidget *value;
@@ -449,6 +497,8 @@ static void do_setup_control_group(EntangleControlPanel *panel,
                              G_CALLBACK(do_update_control_toggle), panel);
             g_signal_connect(control, "notify::value",
                              G_CALLBACK(do_refresh_control_toggle), value);
+            g_signal_connect(control, "notify::readonly",
+                             G_CALLBACK(do_update_control_readonly), value);
             gtk_container_add(GTK_CONTAINER(subbox), value);
         }
     }
@@ -465,7 +515,11 @@ static void do_setup_control_group_ro(EntangleControlPanel *panel,
     int i;
 
     GtkWidget *frame = gtk_expander_new(entangle_control_get_label(ENTANGLE_CONTROL(grp)));
+#if GTK_CHECK_VERSION(3, 2, 0)
+    GtkWidget *subbox = gtk_grid_new();
+#else
     GtkWidget *subbox = gtk_table_new(entangle_control_group_count(grp), 2, FALSE);
+#endif
 
     gtk_container_add(GTK_CONTAINER(frame), subbox);
     gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
@@ -496,7 +550,6 @@ static void do_setup_control_group_ro(EntangleControlPanel *panel,
             int date;
             gchar *text;
 
-            value = gtk_entry_new();
             g_object_get(control, "value", &date, NULL);
             text = g_strdup_printf("%d", date);
             value = gtk_label_new(text);
@@ -544,8 +597,13 @@ static void do_setup_control_group_ro(EntangleControlPanel *panel,
             gtk_misc_set_alignment(GTK_MISC(value), 0, 0);
             gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
             gtk_widget_set_tooltip_text(label, entangle_control_get_info(control));
+#if GTK_CHECK_VERSION(3, 2, 0)
+            gtk_grid_attach(GTK_GRID(subbox), label, 0, i, 1, 1);
+            gtk_grid_attach(GTK_GRID(subbox), value, 1, i, 1, 1);
+#else
             gtk_table_attach(GTK_TABLE(subbox), label, 0, 1, i, i+1, GTK_EXPAND|GTK_FILL, GTK_EXPAND, 3, 3);
             gtk_table_attach(GTK_TABLE(subbox), value, 1, 2, i, i+1, GTK_FILL, GTK_EXPAND, 3, 3);
+#endif
         }
 
     }
