@@ -82,7 +82,7 @@ static void do_entangle_image_display_render_pixmap(EntangleImageDisplay *displa
 
     pw = gdk_pixbuf_get_width(pixbuf);
     ph = gdk_pixbuf_get_height(pixbuf);
-    priv->pixmap = cairo_image_surface_create(CAIRO_FORMAT_RGB24, pw, ph);
+    priv->pixmap = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, pw, ph);
 
     /* Paint the stack of images - the first one
      * is completely opaque. Others are layers
@@ -1025,6 +1025,17 @@ EntangleImageDisplayGrid entangle_image_display_get_grid_display(EntangleImageDi
 }
 
 
+gboolean entangle_image_display_get_loaded(EntangleImageDisplay *display)
+{
+    EntangleImage *image = entangle_image_display_get_image(display);
+    GdkPixbuf *pixbuf = NULL;
+
+    if (image)
+        pixbuf = entangle_image_get_pixbuf(image);
+
+    return pixbuf != NULL;
+}
+
 
 /*
  * Local variables:
