@@ -54,7 +54,6 @@ struct _EntangleSessionBrowserPrivate {
     GtkCellAreaContext *cell_area_context;
 
     GtkCellRenderer *pixbuf_cell;
-    GtkCellRenderer *text_cell;
 
     gulong sigImageAdded;
     gulong sigThumbReady;
@@ -1027,7 +1026,7 @@ entangle_session_browser_queue_draw_item(EntangleSessionBrowser *browser,
     rect.width  = item_area->width  + priv->item_padding * 2;
     rect.height = item_area->height + priv->item_padding * 2;
 
-    if (priv->bin_window)
+    if (priv->bin_window && item_area->width != -1 && item_area->height != -1)
         gdk_window_invalidate_rect(priv->bin_window, &rect, TRUE);
 }
 
@@ -1243,6 +1242,100 @@ entangle_session_browser_button_release(GtkWidget *widget,
     return event->button == 1;
 }
 
+
+static void
+entangle_session_browser_convert_widget_to_bin_window_coords(EntangleSessionBrowser *browser,
+                                                             gint wx,
+                                                             gint wy,
+                                                             gint *bx,
+                                                             gint *by)
+{
+    gint x, y;
+
+    g_return_if_fail(ENTANGLE_SESSION_BROWSER(browser));
+
+    if (browser->priv->bin_window)
+        gdk_window_get_position(browser->priv->bin_window, &x, &y);
+    else
+        x = y = 0;
+
+    if (bx)
+        *bx = wx - x;
+    if (by)
+        *by = wy - y;
+}
+
+
+static void
+entangle_session_browser_set_tooltip_cell(EntangleSessionBrowser *browser,
+                                          GtkTooltip *tooltip,
+                                          EntangleSessionBrowserItem *item)
+{
+    GdkRectangle rect;
+    gint x, y;
+
+    g_return_if_fail(ENTANGLE_SESSION_BROWSER(browser));
+    g_return_if_fail (GTK_IS_TOOLTIP (tooltip));
+
+    rect.x = item->cell_area.x - browser->priv->item_padding;
+    rect.y = item->cell_area.y - browser->priv->item_padding;
+    rect.width  = item->cell_area.width  + browser->priv->item_padding * 2;
+    rect.height = item->cell_area.height + browser->priv->item_padding * 2;
+
+    if (browser->priv->bin_window) {
+        gdk_window_get_position (browser->priv->bin_window, &x, &y);
+        rect.x += x;
+        rect.y += y;
+    }
+
+    gtk_tooltip_set_tip_area (tooltip, &rect);
+}
+
+
+static gboolean
+entangle_session_browser_query_tooltip(GtkWidget  *widget,
+                                       gint        x,
+                                       gint        y,
+                                       gboolean    keyboard_mode G_GNUC_UNUSED,
+                                       GtkTooltip *tooltip,
+                                       gpointer    user_data)
+{
+    g_return_val_if_fail(ENTANGLE_IS_SESSION_BROWSER(widget), FALSE);
+
+    EntangleSessionBrowser *browser = ENTANGLE_SESSION_BROWSER(widget);
+    EntangleSessionBrowserItem *item;
+    GtkCellRenderer *cell = NULL;
+    gint bx = 0;
+    gint by = 0;
+
+    entangle_session_browser_convert_widget_to_bin_window_coords(browser,
+                                                                 x, y,
+                                                                 &bx, &by);
+
+    item = entangle_session_browser_get_item_at_coords(browser,
+                                                       bx, by,
+                                                       FALSE,
+                                                       &cell);
+
+    if (item != NULL) {
+        GtkTreeModel *model = (GtkTreeModel*)user_data;
+        GtkTreeIter iter = item->iter;
+        gchar *file_name = NULL;
+
+        gtk_tree_model_get(model, &iter, FIELD_NAME, &file_name, -1);
+
+        if (file_name) {
+            gtk_tooltip_set_text(tooltip, file_name);
+
+            entangle_session_browser_set_tooltip_cell(browser, tooltip, item);
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+
 static gboolean
 entangle_session_browser_scroll(GtkWidget *widget,
                                 GdkEventScroll *event)
@@ -1556,7 +1649,6 @@ EntangleSessionBrowser *entangle_session_browser_new(void)
 static void entangle_session_browser_init(EntangleSessionBrowser *browser)
 {
     EntangleSessionBrowserPrivate *priv;
-    GdkRGBA fg;
 
     priv = browser->priv = ENTANGLE_SESSION_BROWSER_GET_PRIVATE(browser);
 
@@ -1594,27 +1686,7 @@ static void entangle_session_browser_init(EntangleSessionBrowser *browser)
                  "yalign", 1.0,
                  NULL);
 
-
-    priv->text_cell = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_end(GTK_CELL_LAYOUT(browser), priv->text_cell, FALSE);
-    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(browser),
-                                   priv->text_cell,
-                                   "text", FIELD_NAME,
-                                   NULL);
-
-    fg.red = 1;
-    fg.blue = 1;
-    fg.green = 1;
-    fg.alpha = 1;
-
-    g_object_set(priv->text_cell,
-                 "alignment", PANGO_ALIGN_CENTER,
-                 "wrap-mode", PANGO_WRAP_WORD_CHAR,
-                 "xalign", 0.5,
-                 "yalign", 0.0,
-                 "foreground-set", TRUE,
-                 "foreground-rgba", &fg,
-                 NULL);
+    gtk_widget_set_has_tooltip(GTK_WIDGET(browser), TRUE);
 
     g_signal_connect(priv->model,
                      "row-changed",
@@ -1633,6 +1705,11 @@ static void entangle_session_browser_init(EntangleSessionBrowser *browser)
                      G_CALLBACK (entangle_session_browser_rows_reordered),
                      browser);
 
+    g_signal_connect(GTK_WIDGET(browser),
+                     "query-tooltip",
+                     G_CALLBACK (entangle_session_browser_query_tooltip),
+                     priv->model);
+
     entangle_session_browser_build_items(browser);
     entangle_session_browser_queue_layout(browser);
 
@@ -1995,30 +2072,6 @@ entangle_session_browser_layout_row(EntangleSessionBrowser *browser,
 }
 
 
-static void
-adjust_wrap_width(EntangleSessionBrowser *browser)
-{
-    g_return_if_fail(ENTANGLE_IS_SESSION_BROWSER(browser));
-
-    gint wrap_width = 50;
-
-    /* Here we go with the same old guess, try the icon size and set double
-     * the size of the first icon found in the list, naive but works much
-     * of the time */
-    if (browser->priv->items) {
-        entangle_session_browser_set_cell_data(browser, browser->priv->items->data);
-        gtk_cell_renderer_get_preferred_width(browser->priv->pixbuf_cell,
-                                              GTK_WIDGET(browser),
-                                              &wrap_width, NULL);
-
-        wrap_width = MAX(wrap_width * 2, 50);
-    }
-
-    g_object_set(browser->priv->text_cell, "wrap-width", wrap_width, NULL);
-    g_object_set(browser->priv->text_cell, "width", wrap_width, NULL);
-}
-
-
 static void
 entangle_session_browser_layout(EntangleSessionBrowser *browser)
 {
@@ -2036,9 +2089,6 @@ entangle_session_browser_layout(EntangleSessionBrowser *browser)
         priv->layout_idle_id = 0;
     }
 
-    /* Update the wrap width for the text cell before going and requesting sizes */
-    adjust_wrap_width (browser);
-
     /* Update the context widths for any invalidated items */
     entangle_session_browser_cache_widths(browser);
 
